/* компонент столбчатой диаграммы энергопотребления */

// индексы энергий в датасете графика
const TOTAL = 0;
const ACTIVE = 1;
const REACTIVE = 2;

const primary = getComputedStyle(document.documentElement).getPropertyValue('--primary');
const secondary = getComputedStyle(document.documentElement).getPropertyValue('--secondary');
const grey = getComputedStyle(document.documentElement).getPropertyValue('--grey');
const dark = getComputedStyle(document.documentElement).getPropertyValue('--dark');
const grey_light = getComputedStyle(document.documentElement).getPropertyValue('--grey-light');
Chart.defaults.font.family = 'Montserrat', 'Arial', 'sans-serif';
Chart.defaults.font.size =  getComputedStyle(document.body).fontSize;
Chart.register(ChartDataLabels);
const chartParams = {
    type: 'bar',
    data: {
        labels: ['Сумма', 'Фаза A', 'Фаза B', 'Фаза C'],
        datasets: [
            {
              label: 'Полная',
              data: [],
              backgroundColor: primary,
              borderRadius: 14,
              datalabels: {
                color: dark,
                backgroundColor: primary,
                font: {
                  weight: 'bold'
                }
              }
            },
            {
              label: 'Активная',
              data: [],
              backgroundColor: secondary,
              borderRadius: 14,
              datalabels: {
                color: dark,
                backgroundColor: secondary,
                font: {
                  weight: 'bold'
                }
              }
            },
            {
                label: 'Реактивная',
                data: [],
                backgroundColor: grey,
                borderRadius: 14,
                datalabels: {
                  color: dark,
                  backgroundColor: grey_light,
                  font: {
                    weight: 'bold'
                  }
                }
              },
        ],
    },
    options: {
        aspectRatio: 1/0.36,
        global: {
            defaultFont: 'Montserrat'
        },
        scales: {
          y: {
            afterDataLimits: function(axis) {
              axis.max += axis.max*0.26
            },
            grid: {
              color: grey_light
            }
          },
          x: {
            grid: {
              color: grey_light
            }
          }
        },

        plugins: {
          datalabels: {
            borderRadius: 6,
            padding: {
              top: 10,
              bottom: 10,
              right: 16,
              left: 16
            },
            color: dark,
            align:'top',
            anchor: 'end',
            offset: '10',
          },
          legend: {
              position: 'bottom',
              labels: {
                padding: 36
              }
          },
        }
    },
};

let PowerChart = {
    template: '<canvas id="chart-power"></canvas>',
    props: {
        powerTotal: Object,
        powerActive: Object,
        powerReactive: Object
    },
    mounted() {
        // можно не объявлять в data, т.к. не нужна реактивность
        this.chart = new Chart(this.$el, chartParams);
    },
    watch: {
        powerTotal: function(obj) {
             this. updateChart(obj, TOTAL);
        },
        powerActive: function(obj) {
            this. updateChart(obj, ACTIVE);
        },
        powerReactive: function(obj){
            this. updateChart(obj, REACTIVE);
        },
    },
    methods: {
        updateChart(obj, index) {
            this.chart.data.datasets[index].data[0] = Math.round(obj.total);
            this.chart.data.datasets[index].data[1] = Math.round(obj.phase1);
            this.chart.data.datasets[index].data[2] = Math.round(obj.phase2);
            this.chart.data.datasets[index].data[3] = Math.round(obj.phase3);
            this.chart.update();
        }
    },
};
