from unittest import TestCase

from spread_bolid import commands, errors


class TestFrame(TestCase):
    def test_response_with_error(self, command=None):
        if command is None:
            command = commands.Command(5)

        with self.assertRaises(errors.ModbusError):
            command.of(b'\x05\x80\xFF')

        if isinstance(command, commands._Write):
            with self.assertRaises(errors.WriteError):
                command.of(command.frame[:-1])

    def test_relays_group_read(self):
        command = commands.ReadGroupRelay(3, 14, slave_id=5)
        self.assertEqual(b'\x05\x01\x27\x12\x00\x0E', command.frame)
        self.assertEqual(3, command.start_index)
        self.assertEqual(14, command.count)

        self.assertEqual(
            [False, True, False, False, True, False, False, False, True, False, False, False, False, True],
            command.of(b'\x05\x03\x02\x12\x21')
        )

        self.test_response_with_error(command)

    def test_relays_group_write(self):
        command = commands.SetRelayGroupState(3, [0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1], slave_id=5)
        self.assertEqual(b'\x05\x0F\x27\x12\x00\x0E\x01\x12', command.frame)
        self.assertEqual("Success", command.of(command.frame[:-2]))
        self.test_response_with_error(command)

    def test_relays_single_write(self):
        command = commands.SetRelayState(3, True, slave_id=5)
        self.assertEqual(b'\x05\x05\x27\x12\xFF\xFF', command.frame)
        self.assertEqual("Success", command.of(command.frame))
        self.test_response_with_error(command)

    def test_area_read(self):
        command = commands.ReadAreaState(3, slave_id=5)
        self.assertEqual(b'\x05\x03\x9C\x42\x00\x01', command.frame)
        self.assertEqual(27951, command.of(b'\x05\x03\x9C\x42\x6D\x2F'))
        self.test_response_with_error(command)

    def test_area_write(self):
        command = commands.SetAreaState(3, False, slave_id=5)
        self.assertEqual(b'\x05\x06\x9C\x42\x00\x00', command.frame)

        command = commands.SetAreaState(3, True, slave_id=5)
        self.assertEqual(b'\x05\x06\x9C\x42\x00\x01', command.frame)

        self.assertEqual("Success", command.of(command.frame))
        self.test_response_with_error(command)

    def test_event_desc_length_reading(self):
        command = commands.EventDescLength(slave_id=5)
        self.test_response_with_error(command)

        self.assertEqual(b'\x05\x03\xB4\x46\x00\x01', command.frame)

        self.assertEqual(28, command.of(b'\x05\x03\x02\x00\x1C'))

    def test_max_relay_count_reading(self):
        command = commands.MaxRelayCount(slave_id=5)
        self.assertEqual(b'\x05\x03\xB4\x40\x00\x01', command.frame)

        self.assertEqual(28, command.of(b'\x05\x03\x02\x00\x1C'))
        self.test_response_with_error(command)
