from spread_core.warehouse.warehouse_interface import WarehouseInterface
from spread_dali.address import Group, Broadcast


class DaliWarehouseInterface(WarehouseInterface):
    def __init__(self):
        super(DaliWarehouseInterface, self).__init__()

    def providers_by_address(self, dali_type, short_address):
        for _provider in super(DaliWarehouseInterface, self).providers_by_address(short_address):
            if _provider.dali_type == dali_type:
                yield _provider

    def has_group_provider(self, dali_type, group_id):
        for _ in self.group_providers(dali_type, group_id):
            return True

        return False

    def group_providers(self, dali_type: int, group_id: int):
        for group in self.warehouse.providers_by_address(Group(group_id)):
            if group.dali_type == dali_type:
                yield group

    def broadcast(self, dali_type):
        for broadcast in self.warehouse.providers_by_address(Broadcast()):
            if broadcast.dali_type == dali_type:
                return broadcast

    def add_to_group(self, provider, group_id):
        self.warehouse.include_in_group(group_id, provider)

    def remove_from_group(self, provider, group_id):
        self.warehouse.exclude_from_group(group_id, provider)

    def get_group_size(self, dali_type, group_id):
        count = 0
        for _provider in self.warehouse.group_participants(group_id):
            if _provider.dali_type == dali_type:
                count += 1

        return count

    def get_group_members(self, dali_type, group_id):
        for _provider in self.warehouse.group_participants(group_id):
            if _provider.dali_type == dali_type:
                yield _provider

    def get_all_devices_of_type(self, dali_type):
        for _provider in self.get_all_devices():
            if _provider.is_device and _provider.dali_type == dali_type:
                yield _provider
