import logging

from spread_core.errors.project_errors import CommandError
from spread_core.mqtt.spread.address import EngineryAddress
from spread_core.mqtt.spread.topic import Set, State, Reply
from spread_core.mqtt.spread.variable import Variable
from ..entities import Entity, Funit
from ..recipe import Recipe


class Enginery(Entity):
    def __init__(self, _id: int, **kwargs):
        super(Enginery, self).__init__(_id, **kwargs)
        self._subgineries = []
        self._recipe = None
        self.adapter = None

    @staticmethod
    def parse_args(**kwargs) -> dict:
        return dict(kwargs)

    @property
    def project_id(self):
        return self.adapter.id

    def set_adapter(self, adapter):
        self.adapter = adapter

    @property
    def recipe(self) -> Recipe:
        return self._recipe

    def set_recipe(self, ingredient_type, recipe: Recipe):
        self._recipe = recipe

    def set_subginery(self, subginery):
        if subginery not in self._subgineries:
            self._subgineries.append(subginery)

    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.funit_type.startswith('Enginery'):
            funit_type = topic.entity_address.funit_type.replace('Enginery', '')
        else:
            funit_type = topic.entity_address.funit_type

        self.publish_state(funit_type, None if variable.invalid else variable.value, invalid=variable.invalid)

    def get_address(self, funit_type):
        return EngineryAddress(self.project_id, self.entity_type, self.id, funit_type)

    @classmethod
    def get_address_broadcast(cls, p_id, funit_type='+'):
        return EngineryAddress.broadcast(p_id, cls.__name__, '+', funit_type)

    def publish_state(self, funit_type, value, delay=0, **kwargs):
        funit_class = self.funit_class.of(funit_type)
        invalid = kwargs.get(Variable.INV, False)

        if value is None and funit_class and funit_class.nullable:  # значение None и это разрешено
            if funit_type in self.values:
                if invalid:
                    self.values.pop(funit_type, None)
            else:
                self.set_value(funit_type, value)

            if self.adapter:
                kwargs[Variable.NULLABLE] = funit_class.nullable
                self.adapter.publish(State(self.get_address(funit_type)), Variable(value, **kwargs), retain=True, delay=delay)
        else:
            if value != self.values.get(funit_type, None):
                if value is None:
                    self.values.pop(funit_type, None)
                else:
                    self.set_value(funit_type, value)

                if self.adapter:
                    self.adapter.publish(State(self.get_address(funit_type)), Variable(value, **kwargs), retain=True, delay=delay)

    def publish_reply(self, session_id, value, key=None):
        if self.adapter:
            self.adapter.publish(Reply(self.project_id, session_id), Variable(value, key=key))

    def recipe_for_funit(self, funit):
        return funit, self._recipe

    def publish_provider_command(self, funit_type, value, params=None, sig=None):
        if self.adapter:
            funit, recipe = self.recipe_for_funit(funit_type)
            if recipe is None:
                raise CommandError(f'{self} have not ingredient for {funit_type}`s changing')

            self.adapter.publish(Set(recipe.get_address(funit)), Variable(value, params=params, sig=sig))

    def set_invalid(self):
        for funit_type in self.values.copy():
            self.publish_state(funit_type, None, invalid=True)


class _EngineryFunit(Funit):
    def set(self): self.entity.publish_provider_command(self.funit_type, self.set_value, self.params, self.signature)
