from spread_core.bam.validation.base import corr_par, corr_req_par
import spread_dali as dali
from spread_core.bam.engineries import lighting_area as la


class ButtonPresetValidator:
    def __init__(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong data structure (dict needed)')

        if not corr_req_par('id', int, data) or data['id'] < 1:
            raise ValueError('Wrong id')

        if not corr_req_par('name', str, data):
            raise ValueError('Wrong name')

        if not corr_req_par('isToggle', bool, data):
            raise ValueError('Wrong isToggle')

        if not corr_req_par('handlers', list, data) or len(data['handlers']) != 2 \
                or not isinstance(data['handlers'][0], list) or not isinstance(data['handlers'][1], list):
            raise ValueError('Wrong handlers structure (handlers must be list with two list elements)')

        for handler in data['handlers']:
            for action in handler:
                self.check_action(action)

    def check_action(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong handler action data structure (dict needed)')

        if not corr_req_par('event', str, data) or data['event'] not in dali.device.events.ButtonEvent.STATES.values():
            raise ValueError('Wrong event (must be %s)' % dali.device.events.ButtonEvent.STATES.values())

        if not corr_req_par('command', str, data) or data['command'] not in set(la.VacancyAction.variants
                                                                              + la.OccupancyAction.variants
                                                                              + [
                                                                                  la.ACTION_UP,
                                                                                  la.ACTION_DOWN,
                                                                                  la.ACTION_STEP_UP,
                                                                                  la.ACTION_STEP_DOWN,
                                                                                  la.ACTION_ON_AND_STEP_UP,
                                                                                  la.ACTION_STEP_DOWN_AND_OFF,
                                                                                  la.EMULATE_OCCUPANCY,
                                                                                  la.EMULATE_VACANCY,
                                                                              ]):
            raise ValueError('Wrong command')

        if data['command'] == la.ACTION_SCENE:
            if 'scene' not in data or (data['scene'] not in range(0, 16) and data['scene'] not in ["C1", "C2"]):
                raise ValueError('Wrong scene (must be 0..15, "C1" or "C2")')

        if data['command'] == la.ACTION_LEVEL:
            if not corr_req_par('level', int, data) or data['level'] < 0 or data['level'] > 255:
                raise ValueError('Wrong level (must be 0..255)')

        if data.get('setLightingArea', None) not in ['On', 'Off', None]:
            raise ValueError('Wrong setLightingArea (must be "On", "Off")')

        if not corr_par('stopTuning', bool, data):
            raise ValueError('Wrong stopTuning (must be bool)')
