import re
from spread_core.bam.validation.base import corr_req_par, corr_par


class UserValidator:
    def __init__(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong data structure (dict needed)')

        if not corr_req_par('id', int, data) or data['id'] < 1:
            raise ValueError('Wrong id')

        # key must be GUID
        if not corr_req_par('key', str, data) \
                or not re.match(r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', data['key']):
            raise ValueError('Wrong key')

        if not corr_req_par('login', str, data):
            raise ValueError('Wrong login')

        if not corr_req_par('passwordEncoding', str, data) \
                or data['passwordEncoding'] not in ("None", "Md5", "Sha1", "Sha256", "Sha384", "Sha512"):
            raise ValueError('Wrong passwordEncoding')

        if not corr_req_par('password', str, data):
            raise ValueError('Wrong password')

        if not corr_req_par('rootLocationID', int, data) or data['rootLocationID'] < 1:
            raise ValueError('Wrong rootLocationID')

        if 'homeLocationID' in data:
            if not isinstance(data['homeLocationID'], int) or data['homeLocationID'] < 1:
                raise ValueError('Wrong homeLocationID')
