import logging

import spread_core.mqtt as mqtt
from spread_core.bam.animeo.manager import Animeo
from spread_core.bam.animeo.providers import AnimeoRemote
from spread_core.errors.project_errors import ClassifierError, InitError
from spread_core.tools.service_launcher import Launcher

AnimeoRemote


class AnimeoManagerLauncher(Launcher):
    _dumped = False

    @property
    def manager(self):
        if self._manager is None:
            self._manager = Animeo(self.project_id, self.manager_id)
            self._manager.set_broker(self.mqttc)

        return self._manager

    def on_message(self, mosq, obj, msg):
        super(AnimeoManagerLauncher, self).on_message(mosq, obj, msg)
        try:
            topic = mqtt.topic_of(msg.topic)
            payload = msg.payload
            if isinstance(topic, (mqtt.spread.topic.Set, mqtt.spread.topic.Renew)):
                self._manager.on_command(topic, mqtt.spread.variable.Variable.of(payload))
            elif isinstance(topic, mqtt.spread.topic.State):
                self._manager.on_state(topic, mqtt.spread.variable.Variable.of(payload))
            elif isinstance(topic, mqtt.project.ProjectFile):
                self._manager.on_project(topic=topic, data=payload)
            else:
                logging.debug('[{}] => {}'.format(msg.topic, payload.decode()))
        except ClassifierError as ex:
            logging.error(ex)
        except InitError as ex:
            logging.warning(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    AnimeoManagerLauncher().connect()


if __name__ == '__main__':
    run()
