from .address import EntityAddress

from ..base import Topic, TopicError

STATE = 'State'


class JocketTopic(Topic):
    protocol = 'Jocket'
    topic_type = None

    def __init__(self, p_id):
        self.p_id = int(p_id)

    def __str__(self):
        raise NotImplementedError()

    @classmethod
    def decompose(cls, action, project_id, *parts):

        if action == TopicReply.topic_type:
            return TopicReply(project_id, *parts)

        elif action == TopicState.topic_type:
            return TopicState(project_id, EntityAddress.deserialize(*parts))

        elif action == TopicCommand.topic_type:
            return TopicCommand(project_id, parts[0], EntityAddress.deserialize(*parts[1:]))

        raise TopicError('Unknown action: "%s"' % action)


class TopicReply(JocketTopic):
    topic_type = 'Reply'

    def __init__(self, p_id, session_id):
        super().__init__(p_id)
        self.session_id = session_id

    def __str__(self):
        return f'{self.protocol}/{self.topic_type}/{self.p_id}/{self.session_id}'


class TopicCommand(JocketTopic):
    topic_type = 'Command'

    def __init__(self, p_id, session_id='+', entity_addr='#'):
        super().__init__(p_id)
        self.session_id = session_id
        self.entity_addr = entity_addr

    def __str__(self):
        return f'{self.protocol}/{self.topic_type}/{self.p_id}/{self.session_id}/{self.entity_addr}'


class TopicState(JocketTopic):
    topic_type = 'State'

    def __init__(self, p_id, entity_addr='#'):
        super().__init__(p_id)
        self.entity_addr = entity_addr

    def __str__(self):
        return f'{self.protocol}/{self.topic_type}/{self.p_id}/{self.entity_addr}'
