from . import address
from ..base import Topic, TopicError


class SpreadTopic(Topic):
    protocol = 'Spread'
    action = None

    @classmethod
    def decompose(cls, action: str, *entity_address):
        if Reply.action == action:
            return Reply(*entity_address)

        else:
            entity_address = address.of(*entity_address)

            if Set.action == action:
                return Set(entity_address)

            elif State.action == action:
                return State(entity_address)

            elif Renew.action == action:
                return Renew(entity_address)

        raise TopicError('Unknown action: "%s"' % action)

    def __init__(self, entity_address: address.EntityAddress):
        self.entity_address = entity_address

    def __str__(self):
        return f'{self.protocol}/{self.action}/{self.entity_address}'


class State(SpreadTopic):
    action = 'State'


class Renew(SpreadTopic):
    action = 'Renew'


class Set(SpreadTopic):
    action = 'Set'


class Reply(Topic):
    protocol = SpreadTopic.protocol
    action = 'Reply'

    def __init__(self, project_id, *session_id):
        super(Reply, self).__init__()
        if not session_id:
            raise TopicError('Empty session')

        self.project_id = int(project_id)
        self.session_id = '/'.join(session_id)

    def __str__(self):
        return f'{self.protocol}/{self.action}/{self.project_id}/{self.session_id}'
