from unittest import mock

from spread_core.bam.dali.providers import lights
from spread_core.mqtt.spread.variable import Variable
from spread_core.tests.hardware.dali.test_dali_provider import TestDaliProvider

Tuner = lights.TunerLimited


class TestDaliLight(TestDaliProvider):
    Provider = lights.DaliLight

    @property
    def provider(self) -> lights.DaliLight:
        return self._provider


class TestDimmer(TestDaliLight):
    pass


@mock.patch.object(Tuner, '_start_timer')
class TestFunitTuning(TestDaliLight):
    @property
    def tuner(self):
        return self.provider.tuner

    def _set(self, value: float, speed: float = None):
        params = {'speed': speed} if speed else None

        funit = lights.Tuning(self.provider, Variable(value, params=params))
        funit.set()

    def test_switching_on(self, start):
        self._set(1)
        self.assertIsInstance(self.tuner, Tuner)
        start.assert_called_once()

    @mock.patch.object(Tuner, 'stop')
    def test_switching_off(self, stop, start):
        self.provider._tuner = Tuner(self.provider)
        self._set(0)
        self.assertIsNone(self.tuner)
        stop.assert_called_once()

    def test_speed_applying(self, start):
        self._set(1)
        self._set(1, speed=88.8)
        self.assertEqual(88.8, self.tuner.speed)
        self._set(1, speed=99.9)
        self.assertEqual(99.9, self.tuner.speed)

        start.assert_called_once()

    def test_direction_identical(self, start):
        self._set(1)
        start.assert_called_once()
        start.reset_mock()
        self._set(2)
        self._set(2, speed=99)
        start.assert_not_called()

    def test_direction_switching(self, start):
        self._set(1)
        self.assertIsInstance(self.tuner, Tuner)
        self.assertIsInstance(self.tuner._cmd, Tuner.UP)
        start.assert_called_once()
        start.reset_mock()

        self._set(-1)
        self.assertIsInstance(self.tuner, Tuner)
        self.assertIsInstance(self.tuner._cmd, Tuner.DOWN)
        start.assert_called_once()
        start.reset_mock()

        self._set(0)
        self.assertIsNone(self.tuner)
        start.assert_not_called()
