import spread_knx as knx
from spread_core.bam.knx.providers import Motion
from spread_core.mqtt.spread.variable import Variable

from . import TestKnxMotor

to_position = Motion.to_position
of_position = Motion.of_position


class TestMotion(TestKnxMotor):
    def _set_motion(self, motion, command):
        Motion(self.provider, Variable(motion)).set()
        self.bus.check_bus_message(self, command)

    def _check_level(self, level, motion):
        self._check_state('Motion', motion)
        self._check_state('PositionLevel', level)
        self._check_end_of_publishes()

    def test_0_to_level(self):
        self.assertEqual(10_000, to_position('Close'))
        self.assertEqual(0, to_position('Open'))
        self.assertEqual(None, to_position('Stop'))
        self.assertRaises(ValueError, to_position, 'SomeInvalidMotion')

    def test_0_of_level(self):
        self.assertEqual('Opening', of_position(10, 5))
        self.assertEqual('Closing', of_position(5, 10))
        self.assertEqual('Stopped', of_position(5, 5))

        self.assertEqual('Stopped', of_position(None, 5))
        self.assertEqual(None, of_position(5, None))

    def test_1_set_close(self):
        self._set_motion('Close', knx.commands.Level.set('0/1/0', 254))

    def test_1_set_open(self):
        self._set_motion('Open', knx.commands.Level.set('0/1/0', 0))

    def test_1_set_stop(self):
        self._set_motion('Stop', knx.commands.Step.set('1/0/0'))

    def test_1_set_invalid_motion(self):
        self.assertRaises(ValueError, self._set_motion, 'InvalidMotion', None)
        self.assertRaises(ValueError, self._set_motion, None, None)

    def test_2_get(self):
        self.assertRaises(Motion.NotImplemented, Motion(self.provider, Variable(None)).get)
