import spread_knx as knx
from spread_core.bam.knx.providers import PositionLevel
from spread_core.mqtt.spread.variable import Variable

from . import TestKnxMotor

to_raw = PositionLevel.to_raw
of_raw = PositionLevel.of_raw


class TestPositionLevel(TestKnxMotor):
    def _emulate_response(self, response, level, motion):
        self.provider.on_request_answer(response)
        self._check_level(level, motion)

    def _set_level(self, level, motion):
        PositionLevel(self.provider, Variable(level)).set()
        self.bus.check_bus_message(self, knx.commands.Level.set('0/1/0', to_raw(level)))
        self._check_level(level, motion)

    def _check_level(self, level, motion):
        self._check_state('Motion', motion)
        self._check_state('PositionLevel', level)
        self._check_end_of_publishes()

    def _check_transformation(self, raw_value, level):
        self.assertEqual(raw_value, to_raw(level), msg='PositionLevel.to_raw')
        self.assertEqual(level, of_raw(raw_value), msg='PositionLevel.of_raw')

    def test_0_value_transformations(self):
        self._check_transformation(0, 0)
        self._check_transformation(254, 10_000)
        self._check_transformation(128, 5039)
        self._check_transformation(None, None)

    def test_1_get(self):
        PositionLevel(self.provider, Variable(None)).get()
        self.bus.check_bus_message(self, knx.commands.Level.get('0/0/1'))
        self._emulate_response(knx.commands.Level(1, b'\x00\x80'), 5039, 'Stopped')

    def test_2_set(self):
        self._set_level(10_000, 'Stopped')

    def test_3_stop(self):
        PositionLevel(self.provider, Variable(None)).set()
        self.bus.check_bus_message(self, knx.commands.Step.set('1/0/0', True))
        self._check_end_of_publishes()

    def test_functional(self):
        self._set_level(10_000, 'Stopped')
        self._set_level(0, 'Opening')
        self._emulate_response(knx.commands.Level(1, b'\x00\x00'), 0, 'Stopped')
        self._set_level(5_039, 'Closing')
        self._emulate_response(knx.commands.Level(1, b'\x00\x80'), 5_039, 'Stopped')
        self._emulate_response(knx.commands.Level(1, b'\x00\x80'), 5_039, 'Stopped')
