from spread_core.mqtt import bus
from spread_core.tests.topics.test_common import TestTopic


class TestBusTopic(TestTopic):
    def test_invalid_topic(self):
        self.parse_invalid_topic('Bus')
        self.parse_invalid_topic('Bus/qwe/')
        self.parse_invalid_topic('Bus/State')
        self.parse_invalid_topic('Bus/State/qwe')
        self.parse_invalid_topic('Bus/State/Rapida/1/Dali/2/3/qwe')
        self.parse_invalid_topic('Bus/State/Rapida/1/Dali/2/3')

    def test_send(self):
        topic = self.PARSE('Bus/Send/Rapida/1/Dali/2/3')

        self.assertEqual(topic, bus.TopicBus.of_send('Rapida/1/Dali/2/3'))

        self.assertTrue(topic.is_send)
        self.assertEqual(topic.direction, "Send")
        self.assertEqual(topic.part, 'Rapida/1/Dali/2/3')
        self.assertEqual(topic.funit, '')

    def test_dump(self):
        topic = self.PARSE('Bus/Dump/Rapida/1/Dali/2/3')

        self.assertEqual(topic, bus.TopicBus.of_dump('Rapida/1/Dali/2/3'))

        self.assertTrue(topic.is_dump)
        self.assertEqual(topic.direction, "Dump")
        self.assertEqual(topic.part, 'Rapida/1/Dali/2/3')
        self.assertEqual(topic.funit, '')

    def test_status(self):
        topic = self.PARSE('Bus/State/Rapida/1/Dali/2/3/Status')

        self.assertEqual(topic, bus.TopicBus.of_status('Rapida/1/Dali/2/3'))

        self.assertTrue(topic.is_state)
        self.assertTrue(topic.is_status)

        self.assertEqual(topic.direction, "State")
        self.assertEqual(topic.part, 'Rapida/1/Dali/2/3')
        self.assertEqual(topic.funit, "Status")

    def test_info(self):
        topic = self.PARSE('Bus/State/Rapida/1/Dali/2/3/Info')

        self.assertEqual(topic, bus.TopicBus.of_info('Rapida/1/Dali/2/3'))

        self.assertTrue(topic.is_state)
        self.assertTrue(topic.is_info)

        self.assertEqual(topic.direction, "State")
        self.assertEqual(topic.part, 'Rapida/1/Dali/2/3')
        self.assertEqual(topic.funit, "Info")
