import argparse
import json
import os
import shutil

from spread_core.tools import path


DASHBOARD_STATIC_NAME = 'dashboard_static.json'
DASHBOARD_STATIC_DATA = {
    'httpRoot': 'spread',
    'config_path':  path.CONFIG_PATH,
    'log_path':     path.LOG_PATH,
    'dump_path':    path.DUMP_PATH,
    'sv_path':      path.SV_PATH,
    'lib_path':     path.LIB_PATH,
}

DB_CONF = f"""[fcgi-program:spread_dashboard]
command=daphne -u /run/spread_dashboard.sock --fd 0 --access-log - --proxy-headers spread_dashboard.asgi:application
;process_name=%(program_name)s_%(process_num)02d
numprocs=1
directory=/var/spread
;umask=022
;priority=999
autostart=true
autorestart=true
startsecs=3
startretries=3
;exitcodes=0,2
;stopsignal=TERM
stopsignal=QUIT
stopwaitsecs=10
;stopasgroup=false
;killasgroup=false
user=root
redirect_stderr=false
stdout_logfile={path.LOG(file='dashboard_stdout.log')}
stdout_logfile_maxbytes=3MB
stdout_logfile_backups=10
;stdout_capture_maxbytes=1MB
;stdout_events_enabled=false
stderr_logfile={path.LOG(file='dashboard_stderr.log')}
stderr_logfile_maxbytes=1MB
stderr_logfile_backups=10
;stderr_capture_maxbytes=1MB
;stderr_events_enabled=false
environment=DJANGO_SETTINGS_MODULE=spread_dashboard.settings;SETTINGS={path.CONFIG(file=DASHBOARD_STATIC_NAME)}
;serverurl=AUTO
socket=tcp://localhost:55582
;socket_owner=root
;socket_mode=0700
"""


def run():
    parser = argparse.ArgumentParser(description='Reset spread environment and all projects data')
    parser.add_argument('-y', '--yes', dest='commit', action='store_true', help='Don`t ask commit')
    args = parser.parse_args()

    if not args.commit and input('Are you sure? (y/N)> ').lower() not in ('y', 'yes', 'da'):
        print('Break')
        return

    for root, _, files in os.walk(path.SV_PATH):
        for file in files:
            if file.startswith('spread'):
                os.remove(os.path.join(root, file))
                print('removing ' + os.path.join(root, file))

    shutil.rmtree(path.DUMP_PATH, ignore_errors=True)
    shutil.rmtree(path.CONFIG_PATH, ignore_errors=True)

    configure_dashboard()

    os.system('sudo supervisorctl reread')
    os.system('sudo supervisorctl restart spread_dashboard')
    os.system('sudo supervisorctl update')

    os.system('sudo service mosquitto stop')
    shutil.rmtree(path.LIB('mosquitto'), ignore_errors=True)
    os.system('sudo service mosquitto start')


def configure_dashboard():
    create_dashboard_supervisor()
    create_spread_config()


def create_dashboard_supervisor():
    print(os.path.abspath(os.path.curdir))
    if not os.path.isdir(path.SV_PATH):
        os.mkdir(path.SV_PATH)

    with open(path.SV(file='spread_dashboard.conf'), 'w') as fs:
        fs.write(DB_CONF)


def create_spread_config():
    if not os.path.isdir(path.CONFIG_PATH):
        os.mkdir(path.CONFIG_PATH)

    with open(path.CONFIG(file=DASHBOARD_STATIC_NAME), 'w') as fs:
        json.dump(DASHBOARD_STATIC_DATA, fs, indent=4)


if __name__ == '__main__':
    run()
