import json
import os


def get_subclass(cl, separator):
    if separator(cl):
        return cl
    else:
        arr = cl.__subclasses__()
        if len(arr) > 0:
            for scl in arr:
                if separator(scl):
                    return scl
                else:
                    try:
                        res = get_subclass(scl, separator)
                        if res:
                            return res
                    except:
                        continue
        else:
            return None


def read_file(file_path: str, no_raise=False, binary=False):
    if os.path.exists(file_path):
        with open(file_path, mode='rb') as fs:
            return fs.read() if binary else json.load(fs)

    if no_raise:
        return None

    raise FileNotFoundError(file_path)


def write_file(data, file_path: str):
    if not os.path.exists(os.path.dirname(file_path)):
        os.makedirs(os.path.dirname(file_path), exist_ok=True)

    with open(file_path, mode='w') as fs:
        json.dump(data, fs, indent=2)


def bytes2str(scope: (bytes, bytearray)) -> str:
    """ Returns bytes as HEX-string """
    return scope.hex().upper()


def is_bit(val: int, b: int) -> bool:
    """ Returns True if bit 'b' is set in 'val' """
    if b not in range(0, 8):
        raise ValueError
    return bool(val & (1 << b))


def is_bits(val: int, bits: list) -> bool:
    """ Returns True if bits 'bits' are set in 'val' """
    _r = True
    for _b in bits:
        _r = _r and is_bit(val, _b)
    return _r
