"""Commands and responses from IEC 62386 part 209."""

from __future__ import unicode_literals

from .general import _StandardCommand
from .. import command


class FeatureStatusResponse(command.BitmapResponse):
    bits = ["Automatic Activation"]


class ColourStatusResponse(command.BitmapResponse):
    bits = [
        "xy-coordinate colour point out of range",
        "Colour temperature TC out of range",
        "Auto calibration running",
        "Auto calibration successful",
        "Colour type xy-coordinate active",
        "Colour type colour temperature TC active",
        "Colour type primary N active",
        "Colour type RGBWAF active"
    ]


class ColourTypeFeaturesResponse(command.BitmapResponse):
    bits = ['xy-coordinate capable', 'Colour temperature TC capable']
    ranges = {(2, 4): 'Number of primaries',
              (5, 7): 'Number of RGBWAF channels'}

    def __init__(self, *args, **kwargs):
        super(ColourTypeFeaturesResponse, self).__init__(*args, **kwargs)

    @property
    def primaries(self) -> int:
        return self._value[2:4]

    @property
    def rgb_channels(self) -> int:
        return self._value[5:7]

    @property
    def status(self):
        result = super(ColourTypeFeaturesResponse, self).status

        result.append('Number of primaries is %s' % self.primaries)
        result.append('Number of RGBWAF channels is %s' % self.rgb_channels)

        return result


class _ColourControlCommand(_StandardCommand):
    _devicetype = 8


class Activate(_ColourControlCommand):
    _cmdval = 226


class SetTemperature(_ColourControlCommand):
    _cmdval = 231
    _uses_dtr0 = True
    _uses_dtr1 = True


class SetRgbColour(_ColourControlCommand):
    _cmdval = 235
    _uses_dtr0 = True
    _uses_dtr1 = True
    _uses_dtr2 = True


class SetWafColour(_ColourControlCommand):
    _cmdval = 236
    _uses_dtr0 = True
    _uses_dtr1 = True
    _uses_dtr2 = True


class StoreColourTemperatureLimit(_ColourControlCommand):
    _cmdval = 242
    _uses_dtr0 = True
    _uses_dtr1 = True
    _uses_dtr2 = True
    _sendtwice = True
    COOLEST = 0
    WARMEST = 1
    PHYSICAL_COOLEST = 2
    PHYSICAL_WARMEST = 3


class StoreGearFeaturesStatus(_ColourControlCommand):
    _cmdval = 243
    _uses_dtr0 = True
    _sendtwice = True


class QueryGearFeaturesStatus(_ColourControlCommand):
    _cmdval = 247
    _response = FeatureStatusResponse


class QueryColourStatus(_ColourControlCommand):
    _cmdval = 248
    _response = ColourStatusResponse


class QueryColourValue(_ColourControlCommand):
    _cmdval = 250
    _uses_dtr0 = True
    _response = command.Response
    # _funit_type = ['ColorValue']

    TEMPERATURE = 2
    COOLEST = 128
    PHYSICAL_COOLEST = 129
    WARMEST = 130
    PHYSICAL_WARMEST = 131

    RED = 9
    GREEN = 10
    BLUE = 11
    WHITE = 12
    AMBER = 13
    FREE_COLOUR = 14


class QueryColourTypeFeatures(_ColourControlCommand):
    _cmdval = 249
    _response = ColourTypeFeaturesResponse

    _funit_type = ['ChannelsCount']
