from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import TemplateView

import spread_core.mqtt as mqtt
from spread_core.bam.engineries import light_types
from spread_dashboard.project import bam
from spread_dashboard.project.models import Location, Subginery
from spread_dashboard.utils import app


class Consumption(LoginRequiredMixin, TemplateView):
    template_name = 'consumption/index.html'

    def get(self, request, *args, **kwargs):
        locations = {'locations': []}
        for loc in Location.objects.all():
            node = locations
            for l_id in loc.navigations:
                for _cl in node['locations']:
                    if _cl.get('id') == l_id:
                        cl = _cl
                        break
                else:
                    cl = {
                        'id': l_id,
                        'locations': []
                    }
                    node['locations'].append(cl)

                node = cl

                # node.setdefault('engineries', [])

            try:
                node['id'] = loc.id
                node['name'] = loc.name
                node['engineries'] = [
                    {
                        'type': e.type,
                        'id': e.id,
                        'name': e.v_name,
                        'power': e.attributes.get('power', 0),
                        'renew': str(mqtt.spread.topic.Renew(
                            mqtt.spread.address.EngineryAddress(app().project_id, e.type, e.id, 'Chart')
                        )),
                    } for e in loc.engineries(light_types)
                ]
            except Subginery.DoesNotExist:
                pass

        return self.render_to_response({
            'location': locations['locations'][0],
            'projectName': app().project.name,
            'broker': bam.broker_auth()
        })
