from django.core.management.base import BaseCommand

from spread_dashboard.project.configs import Config
from spread_dashboard.project.models import Header


class Command(BaseCommand):
    help = 'Updates database from local files'

    def add_arguments(self, parser):
        parser.add_argument('--all', action='store_true', help='Does need fill all data')
        parser.add_argument('--check', action='store_true', help='Checking the need for fill')
        parser.add_argument('-c', '--ctp', action='store_true', help='Does need fill a project data')
        parser.add_argument('-t', '--tasks', action='store_true', help='Does need fill a scheduler dump')
        parser.add_argument('-cal', '--calendar', action='store_true', help='Does need fill a calendar dump')
        parser.add_argument('-lf', '--lightfactors', action='store_true', help='Does need fill a light factor dump')
        parser.add_argument('-cs', '--strategies', action='store_true',
                            help='Does need fill a commissioning strategies dump')
        parser.add_argument('-i', '--incubator', action='store_true', help='Does need fill a incubator dump')

    def handle(self, *args, **options):
        if options.get('ctp') or options.get('all'):
            from spread_dashboard.project.bam import Project
            try:
                Project.read_ctp()
            except BaseException:
                print('error ctp filling')
            else:
                print('ctp filling success')

        if str(Config.PROJECT_ID) == '0':
            # После заполнения модули Header в Project.read_ctp() используем project_id
            Config.PROJECT_ID = str(Header.objects.first().project_id)

        if options.get('calendar') or options.get('all'):
            from spread_dashboard.scheduler.models import Holiday
            if (options.get('check') and not bool(Holiday.objects.first())) or not options.get('check'):
                try:
                    Holiday.fill()
                except BaseException:
                    print('error calendar filling')
                else:
                    print('calendar filling success')

        if options.get('tasks') or options.get('all'):
            from spread_dashboard.scheduler.models import SchBase, SchLocation
            if (options.get('check') and not bool(SchLocation.objects.first())) or not options.get('check'):
                try:
                    SchBase.fill()
                except BaseException:
                    print('error tasks filling')
                else:
                    print('tasks filling success')

        if options.get('lightfactors') or options.get('all'):
            from spread_dashboard.scheduler.models import LightFactor
            if (options.get('check') and not bool(LightFactor.objects.first())) or not options.get('check'):
                try:
                    LightFactor.fill()
                except BaseException:
                    print('error light factors filling')
                else:
                    print('light factors filling success')

        if options.get('strategies') or options.get('all'):
            from spread_dashboard.scheduler.models import CommissioningStrategy
            if (options.get('check') and not bool(CommissioningStrategy.objects.first())) or not options.get('check'):
                try:
                    CommissioningStrategy.fill()
                except BaseException:
                    print('error commissioning strategies filling')
                else:
                    print('commissioning strategies filling success')

        if options.get('incubator') or options.get('all'):
            from spread_dashboard.project.models import Cycle
            if (options.get('check') and not bool(Cycle.objects.first())) or not options.get('check'):
                try:
                    Cycle.fill()
                except BaseException as exc:
                    print(f'error incubator cycles filling: {exc}')
                else:
                    print('incubator cycles filling success')
