import datetime
import json
import sys

from django import http
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.translation import gettext as _

import paho.mqtt.client as mqtt

from spread_core.bam.mercury.providers import MercuryElectricMeter
from spread_dashboard.project import bam
from spread_dashboard.responses import Success, Error


@login_required
def page(request, meter_id=0):
    try:
        if meter_id:
            meter = bam.models.Provider.objects.get(id=meter_id, type=MercuryElectricMeter.__name__)
            stack = {meter.id: meter.name or f'id: {meter.name}'}
        else:
            stack = {id_: name or f'id: {id_}' for (id_, name) in
                     bam.models.Provider.objects.filter(type=MercuryElectricMeter.__name__).values_list('id', 'name')}
            if 'id' in request.GET:
                meter_id = int(request.GET.get('id'))
            elif len(stack) > 0:
                meter_id = list(stack.keys())[0]

            if meter_id:
                meter = bam.models.Provider.objects.get(id=meter_id, type=MercuryElectricMeter.__name__)
            else:
                meter = None

        if meter:
            if not meter.name:
                meter.name = f'id: {meter.id}'

            config = bam.config(meter)['project']
            context = dict(
                config=config,
                meter=meter,
                stack=stack,
                skin=str(request.GET.get('skin')).lower()
            )
        else:
            context = {}

        return render(request, 'mercury/index.html', context=context)
    except bam.models.Manager.DoesNotExist as ex:
        return http.HttpResponse('<h1> ' + str(ex) + ' </h1>')


@login_required
def on_api(request, api_name):
    if hasattr(sys.modules[__name__], api_name):
        try:
            return getattr(sys.modules[__name__], api_name)(request)
        except BaseException as ex:
            print(str(ex))
            return http.HttpResponseServerError(str(ex))

    return http.HttpResponseNotAllowed(api_name)


@login_required
def check_password(request):
    _id = int(request.GET.get('id'))
    _password = request.GET.get('password')
    load = bool(request.GET.get('loadon'))
    model = bam.models.Provider.objects.get(id=_id, type=MercuryElectricMeter.__name__)
    if model.attributes['password2'] == _password:
        # сами шлем запрос на счетчик
        meter = bam.models.Provider.objects.get(id=_id, type=MercuryElectricMeter.__name__)
        config = bam.config(meter)
        set_topic = config['project'].get('set')
        set_topic = set_topic.replace('{}', 'LoadOn')
        params = {'value': not load, 'params': {'password': _password}}
        #TODO запрос на АУФ
        mqtt_client = mqtt.Client()
        mqtt_client.username_pw_set(config['broker']['login'], config['broker']['password'])
        mqtt_client.connect('localhost')
        mqtt_client.publish(set_topic, json.dumps(params))
        mqtt_client.disconnect()
        return Success()
    return Error(_('mercury_password_invalid'))


def config_api(request):
    meter = request.GET.get('meter')
    meter = int(meter) if meter.isdigit() else None
    if meter:
        meter = bam.models.Provider.objects.get(id=int(meter), type=MercuryElectricMeter.__name__)
    else:
        meter = bam.models.Provider.objects.filter(type=MercuryElectricMeter.__name__)[0]
    config = bam.config(meter)['project']
    now = datetime.datetime.now()

    MONTHS = [None, _('january'), _('february'), _('march'), _('april'),
              _('may'), _('june'), _('july'), _('august'), _('september'),
              _('october'), _('november'), _('december')]

    periods = {'total': [{'period': 0, 'text': '', 'month': 0}],
              'year': [{'period': 1, 'text': now.year, 'month': 0}, {'period': 2, 'text': now.year - 1, 'month': 0}],
              # берем 12 месяцев с текущего в обратном порядке
              'month': [{'month': MONTHS.index(month), 'text': f'{now.year} {month}', 'period': 3}
                        for month in MONTHS[now.month:0:-1]] +
                         [{'month': MONTHS.index(month), 'text': f'{now.year - 1} {month}', 'period': 3}
                          for month in MONTHS[-1:now.month:-1]],
              'day': [{'period': 4, 'text': _('mercury_current_day'), 'month': 0}, {'period': 5, 'text': _('mercury_prev_day'), 'month': 0}]
              }

    result = {
        'config': config,
        'periods': periods,
    }

    response = http.HttpResponse(content=json.dumps(result))
    return response
