import logging
import os
import re
import shutil
import signal

from django.http import HttpResponse
from django.shortcuts import render, redirect

from spread_dashboard.auth2.decorators import staff_required
from spread_dashboard.utils import app
from . import forms, configs, bam
from spread_core.tools import path


@staff_required
def config(request, file_name):
    if file_name == 'info':
        return render(request, 'project/info.html', context=dict(frontier_active=configs.Frontier.exist()))
    i = re.search(r'(_\d+)+', file_name)
    args = []
    if i:
        args.append(i[0])
    if request.method == 'POST':
        args.append(request.POST)

    # Получить форму по имени конфига
    forms_by_name = forms.get_forms_by_file_name()
    form = forms_by_name.get(file_name, None)
    for form_name_item, form_item in forms_by_name.items():
        if re.sub(r'[\d_]', '', file_name) == re.sub(r'[\d_]', '', form_name_item):
            form = form_item
    form = form.of(*args)

    if request.method == 'POST':
        if form.is_valid():
            if form.changed_data:
                form.save()
                try:
                    if isinstance(form, forms.DashboardForm):
                        app().settings.read()
                        logging.info('config updated')
                    else:
                        os.system('sudo supervisorctl signal %s %s' % (int(signal.SIGHUP), form.instance.service_name))
                except BaseException as ex:
                    logging.error(ex)

            return redirect(request.path.split(file_name)[0] + '#' + form.instance.config_dest + '_' + file_name)

    return render(request, 'project/config.html', dict(form=form, name=form.__class__.Meta.model.__name__))


@staff_required
def ctp(request):
    with open(configs.Projector.ctp_file, 'wb') as dest:
        dest.write(request.body)

    bam.Project.generate_configs()

    return redirect(request.path.split('ctp')[0])


@staff_required
def reread_ctp(request):
    try:
        forced = request.GET.get('forced', '').lower() == 'true'
        bam.Project.generate_configs(force=forced)
        if forced:
            return HttpResponse("Configs recreated\n")
        else:
            return HttpResponse("Configs updated\n")
    except BaseException as ex:
        return HttpResponse(str(ex) + "\n")


@staff_required
def index(request):
    if app().restarted:
        return render(request, 'project/redirector.html', dict(timeout=15, href=request.path))
    elif request.method == 'POST':
        if 'download' in request.POST:
            file = configs.Projector.ctp_file
            if os.path.exists(file):
                with open(file, 'rb') as fh:
                    response = HttpResponse(fh.read(), content_type="application/force-download")
                    response['Content-Disposition'] = 'inline; filename=' + os.path.basename(file)
                    return response
        elif 'ctp' in request.FILES:
            with open(configs.Projector.ctp_file, 'wb+') as dest:
                for chunk in request.FILES['ctp'].chunks():
                    dest.write(chunk)

            bam.Project.generate_configs()

        elif 'clear' in request.POST:
            _clear_configs(request)

        elif 'supervisor' in request.POST:
            _supervisor(request)

        return redirect(request.path)
    elif 'set_frontier' in request.GET:
        use_frontier = request.GET.get('is_active', False)
        frontier = configs.Frontier()
        events_sqlite = configs.EventsSqlite()
        if use_frontier:
            frontier.save(generate_sv=True)
            events_sqlite.packetFormat = events_sqlite.FORMAT_TROS3
        else:
            frontier.delete()
            events_sqlite.packetFormat = events_sqlite.FORMAT_SPREAD

        events_sqlite.save()
        os.system('sudo supervisorctl signal %s %s' % (int(signal.SIGHUP), events_sqlite.service_name))
        os.system('sudo supervisorctl reread')
        os.system('sudo supervisorctl update')

        return redirect(request.path)

    return render(request, 'project/project.html', dict(
        tree=configs.Tree(),
        uncheckable=[configs.Common.file]
    ))


def _supervisor(request):
    services = request.POST.get('supervisor', '').split(',')
    if services:
        self_name = 'spread_%s' % configs.Dashboard.file
        if 'restart' in request.POST and '_%s' % configs.Dashboard.file in services:
            app().project_id = 0
            app().restarted = True
        if 'restart' in request.POST or 'stop' in request.POST:
            def sorter(s):
                try:
                    return (configs.Equipment.config_dest, configs.DaliManager.config_dest,
                            configs.Common.config_dest, configs.TcpAdapter.config_dest,
                            configs.EventsSqlite.config_dest).index(s.split('_')[1])
                except:
                    return 999

            arr = sorted([('spread_%s' % i).replace('__', '_') for i in services], key=sorter)
            extras = ('spread_%s_%s' % (configs.Equipment.config_dest, configs.LightingEquipment.file),
                      'spread_%s_%s' % (configs.Equipment.config_dest, configs.ShadingEquipment.file),)
            for extra in extras:
                if extra in arr:
                    arr.remove(extra)
                    arr.insert(0, extra)

            if self_name in arr:
                arr.remove(self_name)
            if arr:
                os.system('sudo supervisorctl stop %s' % ' '.join(arr))

        if 'restart' in request.POST:
            def sorter(s):
                try:
                    return (configs.TcpAdapter.config_dest, configs.EventsSqlite.config_dest,
                            configs.Common.config_dest, configs.Equipment.config_dest,
                            configs.DaliManager.config_dest).index(s.split('_')[1])
                except:
                    return 999

            arr = sorted([('spread_%s' % i).replace('__', '_') for i in services], key=sorter)

            extras = []
            for extra in ('spread_%s' % configs.Projector.file,):
                if extra in arr:
                    extras.append(extra)
                    arr.remove(extra)

            if extras:
                os.system('sudo supervisorctl start %s' % ' '.join(extras))

            if self_name in arr:
                restart_dashboard = True
                arr.remove(self_name)
            else:
                restart_dashboard = False

            os.system('sudo supervisorctl reread')
            if arr:
                os.system('sudo supervisorctl start %s' % ' '.join(arr))
            os.system('sudo supervisorctl update')
            if restart_dashboard:
                os.kill(os.getpid(), signal.SIGTERM)


def _clear_configs(request):
    files_name = configs.get_files_name(supervisor=True)
    files_dest_name = configs.get_files_name()
    if request.POST.get('clear_data', False):
        shutil.rmtree(path.DUMP(app().project_id), ignore_errors=True)

        if os.path.exists(configs.Projector.ctp_file):
            os.remove(configs.Projector.ctp_file)
    for file in os.listdir(path.SV_PATH):
        dest = configs.path.SV(file=file)
        if os.path.isfile(dest):
            for file_name in files_name:
                if file_name in file:
                    os.remove(dest)
    for file in os.listdir(path.CONFIG_PATH):
        dest = path.CONFIG(file=file)
        if os.path.isfile(dest):
            if file.rsplit('.json', 1)[0] in files_dest_name:
                os.remove(dest)
        else:
            shutil.rmtree(dest, ignore_errors=True)
    os.system('sudo supervisorctl reread')
    os.system('sudo supervisorctl update')
    os.system('sudo service mosquitto stop')
    mosquitto = configs.Projector().mosquittoDB
    if os.path.isdir(mosquitto):
        shutil.rmtree(mosquitto, ignore_errors=True)
    elif os.path.isfile(mosquitto):
        os.remove(mosquitto)
    os.system('sudo service mosquitto start')
    app().project_id = 0
