let chartData = {
    labels: [],
    periods: null,
};

let addPoint = false;  // глобальный флаг добавления точки начала периода к графику

function requestChartData(id, topic, from, to, place) {
    /* Запрашивает данные потребления как по локации,
    так и по отдельным потребителям в световой зоне
     */
    if (!['location', 'area'].includes(place)) {
        throw 'Неверное значение параметра place: ' + place + '. Укажите "location" или "area"';

    }
    chartData.periods = chartData.periods ?? calculatePeriods(id, from, to);
    const algo = "integral";

    chartData.periods.forEach(function(period, i) {
        client.send(topic,
            JSON.stringify({
                    value: {
                        algorithm: algo,
                        startTime: period.from,
                        endTime: period.to,
                    },
                    key: [id, i, period.label, place+"_chart_data"].join('#'),
                    session: broker.guid
                }
            ));
    });
}

function calculatePeriods(id, fromDate, toDate) {
    let toDateObj = moment(toDate);
    let fromDateObj = moment(fromDate);
    let nowObj = moment();

    let delta = moment.duration(toDateObj.diff(fromDate));
    //TODO формат зависит от локали
    let quant = moment.duration(1, 'days');
    let format = 'D.MM';
    if (delta.asDays() < 2) {
        quant = moment.duration(1, 'hours');
        format = 'k ч. D.MM';
        if (delta.asDays() <= 1) {
            format = 'k ч.';
        }
    } else if (delta.asDays() <= 31) {
        // оставим параметры по умолчанию и выйдем
    } else if (delta.asDays() < 70) {
        quant = moment.duration(3, 'days');
    } else if (delta.asWeeks() < 52) {
        quant = moment.duration(1, 'weeks');
    } else {
        quant = moment.duration(1, 'months');
        format = "MM'YY";
    }
    if (fromDateObj.year() != toDateObj.year()) {
        format += '.YY';
    }
    let periods = [];
    if (quant.asDays() < 1 && toDateObj.isAfter(nowObj)) {
        toDateObj = nowObj.endOf('hour');
    } else {
        toDateObj = toDateObj.endOf('day');
    }

    if (quant.asHours() == 1) {
        do {
            let from = fromDateObj.format();
            fromDateObj.add(quant);
            let to = fromDateObj.format();
            let label = fromDateObj.format(format);
            periods.push({from: from, to: to, label: label});
        } while (fromDateObj.isBefore(toDateObj))

    } else {
        do {
            let label = toDateObj.format(format);
            chartData.labels.push(label);
            periods.push({to: toDateObj.format(), label: label, from: toDateObj.subtract(quant).format()});  // from.add() меняет сам объект
        } while (toDateObj.isAfter(fromDateObj))
        chartData.fromDate = quant.asDays() != 1 && fromDateObj.format(format);
        // С вычитанием последнего кванта мы можем перевалить за границу интервала, поэтому обрежем последний период
        if (chartData.fromDate) {
            periods[periods.length-1].from = fromDateObj.format();
        }
        periods.reverse();
        chartData.labels.reverse();
    }
    // данные агрегируются по квантам в точке конечной даты кванта
    // перед созданием графика добавим начало периода на оси со значениями 0, для наглядности
    addPoint = chartData.fromDate && chartData.fromDate != chartData.labels[0];
    if (addPoint) {
        chartData.labels.splice(0, 0, chartData.fromDate);
    }

    return periods
}

function drawConsumptionChart(id, fromDate) {
    let chartName = 'chart-' + id;
    // при изменении периода удалим старый график, чтоб на его месте построить новый
    $("#" + chartName)?.remove();
    // Create Chart Object
    let chartObj = new Dopyo(chartName, 'Line');
    formatLabels();
    chartObj.setLabels(chartData.labels);
    // если нет withoutAwada, то светильники невалидные, скроем график целиком
    if (!arraySum(chartData[id].legends.withoutAwada.values)) {
        $('#chart-container-' + id).attr('hidden', true);
        return;
    }
    // Set legends and values
    for (let key in chartData[id].legends) {
        let legend = chartData[id].legends[key];
        clearEmptyData(legend.values);
        // добавим 0 для точки начала периода
        if (addPoint) {
            legend.values.splice(0, 0, 0);
        }
        chartObj.addLegend(legend);
    }
    formatChart(chartObj);
    // Inject chart into DOM object
    let div = document.getElementById('consumption-chart-' + id);

    // Draw
    if (chartData[id]?.legends.awada.values.length > 0) { // && arraySum(chartData[id]?.legends.awada.values)
        $('#chart-container-' + id).attr('hidden', false);
        chartObj.inject(div);
        chartObj.draw(false);
    } else {
        // прячем <div> с графиком
        $('#chart-container-' + id).attr('hidden', true);
        //progressElement.hidden = true;
        return;
    }
    // Считаем и отображаем экономию
    let cons_elem = document.getElementById("consumption-" + id);
    let cons = document
        .querySelector('[total-power-' + id + ']')
        .textContent;

    cons_elem.textContent = cons + ' ' + gettext('consumption_energy_unit');
    document.getElementById('economy-' + id).textContent = calculateSaving(id) + ' %';
    delete chartData[id];
}

function processChartData(key, data) {
    let [id, index, label, place] = key.split('#');
    // вычислили число периодов, столько точек будем ждать из брокера
    chartData[id].valueCount = chartData[id]?.valueCount || chartData.periods.length * (chartData[id]?.partCount || 1);

    if (Object.keys(data).length === 0) {
        // нет данных, точку не обрабатываем
        if (chartData[id].valueCount) {
            --chartData[id].valueCount;
        }
        return;
    }
    // if (index % 2 != 0) {
    //     label = '';
    // }
    if (!chartData.labels[index]) {
        chartData.labels[index] = label;
    }

    let newValue = parseFloat(chartData[id].legends.awada.values[index] || 0.0) + (data.value/1000 || 0.0);
    let maxValue = consumptionWithoutAwada(id, index, data, place.includes('area'));
    chartData[id].legends.withoutAwada.values[index] = maxValue;
    // данные могут быть неточны и превышать максимум. такие значения приравняем к максимуму в соответствующей точке
    if (newValue > maxValue) {
        chartData[id].legends.awada.values[index] = maxValue;
    } else {
        chartData[id].legends.awada.values[index] = +newValue.toFixed(3);
    }
    --chartData[id].valueCount;
    if (chartData[id].valueCount <= 0) {
        drawConsumptionChart(id);
    }
}

function clearEmptyData(list) {
    for (let i=0; i<list.length; i++) {
        if (list[i] === undefined) {
            list[i] = 0;
        }
    }
    return list;
}

function consumptionWithoutAwada(id, index, data, isArea = false) {
    let totalPower = +document
        .getElementById(id)
        .getAttribute("total-power-" + id);
    if (!totalPower) {
        return 0.0;
    }
    let hours = ((data.workTime || 0.0) + (data.idleTime || 0.0));
    // для световой зоны определяем, является ли текущее значение максимальным из всех потребителей
    if (isArea) {
        hours = chartData[id].hoursWithoutAwada[index] = Math.max(hours, chartData[id].hoursWithoutAwada[index] || 0);
    }
    return +(totalPower * hours / (60*60*1000*1000)).toFixed(3);
}

function resetChartData(id=null, partCount=1) {
    if (id) {
        chartData[id] = {
            valueCount: 0,  // число число значений, ожидаемых для построения графика
            partCount: partCount,  // число слагаемых в каждой точке (напр. светильников в световой зоне)
            legends: {
                withoutAwada: {"name": "Без AWADA", "stroke": "#B1B9FF", "fill": "#fff", "values": []},
                awada: {"name": "С AWADA", "stroke": "#A700FF", "fill": "#fff", "values": []}
            },
            hoursWithoutAwada: []  // массив для хранения максимального из времен работы потребителей в световой зоне
        };
    } else {
        chartData = {
            labels: [],
            periods: null,
        };
    }

}

function calculateSaving(id) {
    const noAwadaValues = chartData[id].legends?.withoutAwada?.values;
    const noAwadaConsumption = noAwadaValues ? arraySum(noAwadaValues) : 0;
    let totalPower = +document
        .getElementById(id)
        .textContent;
    return noAwadaConsumption ? ((1 - (+totalPower / noAwadaConsumption))*100).toFixed(1) : 0;
}

function formatChart(chartObject) {
    // Вертикальная ось - Х, а горизонтальная Y ¯\_(ツ)_/¯
    chartObject.setLineStyle('straight');
    chartObject.setPadding(50, 20, 50, 10);
    chartObject.setTitleShow(false);
    chartObject.setLegendTableShow(false);
    chartObject.setGridXShow(true);
    if (chartObject.getMaxValue() < 10) {
        chartObject.setGridXInterval(1);
    }
    if (chartObject.getMaxValue() < 1) {
        chartObject.setGridXInterval(0.5);
    }
    if (chartObject.getMaxValue() < 0.5) {
        chartObject.setGridXInterval(0.1);
    }
    if (chartObject.getMaxValue() < 0.1) {
        chartObject.setGridXInterval(0.05);
    }
    chartObject.setGridYLabelShow(true);
    chartObject.setTooltipShow(true);
    chartObject.setGridYShow(true);
    chartObject.setAnimateShow(false);
}

function formatLabels() {
    let len = chartData.labels.length
    if (len > 16 && len % 3 == 1) {
        for (let i = 1; i < len; i++) {
            if (i % 3 != 0) {
                chartData.labels.splice(i, 1, '');
            }
        }
    } else if (len > 24) {
        for (let i = 1; i < len; i++) {
            if (i % 4 != 0) {
                chartData.labels.splice(i, 1, '');
            }
        }
    } else if (len > 14) {
        for (let i = 1; i < len; i += 2) {
            chartData.labels.splice(i, 1, '');
        }
    }
}

function arraySum(array) {
    return array.reduce(function(sum, item) {
        if (item != undefined) {
            return sum + parseFloat(item);
        }
        return 0;
    }, 0);
}

function hideCharts() {
    // удаляет графики при обновлении страницы
    $('svg')
        .filter(function () {
            return this.id.match(/chart-\d+/);
        }).remove();

    $('span')
        .filter(function () {
            return this.id.match(/consumption-\d+/);
        }).text('');

    $('span')
        .filter(function () {
            return this.id.match(/economy-\d+/);
        }).text('');
}
