// компонент формы редактирования цикла
let EditCycleModal = {
    delimiters: ['{|', '|}'],
    props: {
        cycle: Object,
    },
    inject: ['copyJson', 'addOrEditCycle', 'getNextTriggerIndex'],
    data() {
        return {
            cycleCopy: {},
            triggers: {},
            currentDay: {value: 0, displayValue: 1},
            editModal: undefined,
            dayForCopy: {displayValue: 2, value: 1}, // переменная для хранения дня, из которого можно скопировать триггеры
        }
    },
    computed: {
        cycleDays: function () {
            // генерим дни по длине периода, а не фактическим данным в объекте triggers
            const len = this.cycleCopy.period || 0;
            let days = Array.from(
                {length: len},
                (_, i) => ({value: i, displayValue: i + 1})
            )
            return days;
        },
        dayTriggers: function () {
            // возвращаем entries, чтоб проще удалять триггеры по ид
            let vm = this;
            let triggers = vm.triggers || {};
            triggers = Object
                .entries(triggers)
                ?.filter(it => it ? it[1].day == vm.currentDay.value : false)
            return triggers;
        },
    },
    methods: {
        getDayTriggers(dayValue) {
            // тут возвращаем массив values
            let vm = this;
            let triggers = vm.triggers || {};
            triggers = Object
                .values(triggers)
                ?.filter(it => it ? it.day == dayValue : false)
            return triggers;
        },
        getDayChartData(cycle, dayNumber) {
            if (dayNumber >= cycle.period) {
                console.error('Указанный номер дня не входит в цикл');
                return {};
            }

            let triggers = this.dayTriggers?.map(it => it[1]) || [];
            triggers = triggers.sort((a, b) => a.time > b.time ? 1 : -1);
            triggers = triggers.filter(item => item.time);
            // добавляем фиктивные точки в начало и конец набора, для красоты
            let start = {time: '00:00', level: this.getPreviousLevel(dayNumber)}
            // let start = this.getStartTriggerByChart(dayNumber);
            let end = {time: '23:59', level: triggers[triggers.length - 1]?.level || 0}
            triggers.unshift(start);
            triggers.push(end);
            // Нужно изменить массив треггеров таким образом, что бы каждый триггер стал двумя точками (началом триггера и концом)
            // где начало имеет время триггера, а конец - время триггера + длительность
            // и начало имеет уровень предыдущего триггера, а конец - уровень текущего
            let chartData = [];
            //Для созданных триггеров
            for (let i = 0; i <= triggers.length - 1; i++) {
                let trigger = triggers[i];
                if (i === 0 ) {
                    chartData.push(...this.getStartTriggerByChart(dayNumber));
                    continue;
                }
                let nextTrigger = triggers[i + 1] || {time: '23:59', level: 0};
                let beforeTrigger = triggers[i - 1];
                chartData.push({time: trigger.time, level: beforeTrigger.level}); // start
                let endTime = moment(trigger.time, 'HH:mm').add(this.cycleCopy.duration, 'minutes');
                let nextStartTime = moment(nextTrigger.time, 'HH:mm');
                if (endTime < nextStartTime) {
                    chartData.push({time: endTime.format('HH:mm'), level: trigger.level}) // end;
                }
                else {
                    let ratio = (this.cycleCopy.duration - endTime.diff(nextStartTime, 'minutes')) / this.cycleCopy.duration;
                    let endLevel = Number(beforeTrigger.level) + (trigger.level - beforeTrigger.level) * ratio
                    chartData.push({time: nextTrigger.time, level: endLevel}) // end;
                    triggers[i] = {...triggers[i], time: nextTrigger.time, level: endLevel};
                }
            }

            chartData = chartData.filter((item, index) => index === 0 || this.cycleCopy.duration !== 0 || (item.time !== chartData[index - 1].time));
            return chartData;
        },
        parseCycleChartLabel(dayValue, timeString) {
            //ЭТО возможный хэндлер для оси Х графика полного цикла
            // переводим время в минуты и делим на число минут в сутках
            // добавим номер дня = итоговое значение точки
            const sutki = 1440; // число минут в сутках
            const oneHour = 60; // число минут в часе
            let [hour, minutes] = timeString.split(':');
            minutes = parseInt(hour) * oneHour + parseInt(minutes);  // число минут
            return dayValue + minutes / sutki;
        },
        getCycleChartData(cycle) {
            if (!this.triggers) {
                return [];
            }
            let triggers = Object
                .values(this.triggers)
                ?.filter(value => value.day < cycle.period && value.time)  // берем только триггеры, относящиеся к заданному периоду
                ?.sort((a, b) => { // сортируем по дню и времени
                    if (parseInt(a.day) > parseInt(b.day)) {
                        return 1;
                    } else if (parseInt(a.day) < parseInt(b.day)) {
                        return -1;
                    } else if (a.time >= b.time) {
                        return 1;
                    }
                    return -1;
                })
                ?.map(value => ({  // форматируем датасет для удобства передачи в график
                    date: this.parseCycleChartLabel(parseInt(value.day) + 1, value.time), //`${parseInt(value.day)+1} ${value.time}`
                    level: value.level,
                }));
            let start = {date: 1, level: this.cycleCopy.startLevel || 0}; //{date: '1 00:00', level: this.cycleCopy.startLevel || 0}
            triggers.unshift(start);
            let last = triggers[triggers.length - 1];
            const magicConst = 0.9985;  // эта константа выведеа имперически ведущми константоводами
            let end = {date: parseInt(this.cycleCopy.period) + magicConst, level: last.level || 0};  //{date: `${this.cycleCopy.period || 1} 23:59`, level: last.level || 0};
            triggers.push(end);
            let chartData = [];
            const minutesInDay = 24 * 60;
            const duration = this.cycleCopy.duration / minutesInDay;
            //Для созданных триггеров
            for (let i = 0; i <= triggers.length - 1; i++) {
                let trigger = triggers[i];
                if (i === 0 || i === triggers.length - 1) {
                    chartData.push({date: trigger.date, level: trigger.level});
                    continue;
                }
                let nextTrigger = triggers[i + 1] || {date: this.cycleCopy.period, level: 0};
                let beforeTrigger = triggers[i - 1];
                chartData.push({date: trigger.date, level: beforeTrigger.level}); // start
                let endTime = trigger.date + duration;
                let nextStartTime = nextTrigger.date
                if (endTime < nextStartTime) {
                    chartData.push({date: endTime, level: trigger.level}) // end;
                }
                else {
                    let ratio = (duration - (endTime -nextStartTime)) / duration;
                    let endLevel = Number(beforeTrigger.level) + (trigger.level - beforeTrigger.level) * ratio
                    chartData.push({date: nextTrigger.date, level: endLevel}) // end;
                    triggers[i] = {...triggers[i], date: nextTrigger.date, level: endLevel};
                }
            }

            chartData = chartData.filter((item, index) => index === 0 || this.cycleCopy.duration !== 0 || item.date !== chartData[index - 1].date);
            return chartData;
            // return triggers;
        },
        getPreviousLevel(day) {
            // возвращает уровень последнего тригера за предыдущие дни, для красоты отрисовки графиков
            let prevTrigs = [];
            while (prevTrigs.length == 0) {
                if (day <= 0) {
                    return this.cycleCopy.startLevel || 0;
                }
                day--;
                prevTrigs = this.getDayTriggers(day);
            }
            let lastTrig = prevTrigs.sort((a, b) => {
                return a.time > b.time ? 1 : -1
            }).pop();
            return lastTrig.level;
        },
        getStartTriggerByChart(day) {
            // возвращает первый триггер для переванного дня, для красоты отрисовки графиков
            let prevTrigs = [];
            let dayCount = day;
            while (prevTrigs.length == 0) {
                if (dayCount <= 0) {
                    return [{time: '00:00', level:this.cycleCopy.startLevel || 0}]
                }
                dayCount--;
                prevTrigs = this.getDayTriggers(dayCount);
            }
            prevTrigs = prevTrigs.sort((a, b) => {
                return a.time > b.time ? 1 : -1
            });
            let triggersByDay = this.getDayTriggers(day);
            let triggerByDay = triggersByDay.sort((a, b) => {
                return a.time > b.time ? 1 : -1
            })
            let firstTrig = triggerByDay[0] ?? {time: '23:59', level: 0};
            let lastTrig = prevTrigs.pop();
            let predLastTrigLevel = prevTrigs.pop()?.level ?? this.getPreviousLevel(lastTrig.day);
            let timeEndLastTrig = moment(lastTrig.time, 'HH:mm').add(this.cycleCopy.duration, 'minutes')

            if (timeEndLastTrig < moment('23:59', 'HH:mm') || (day - 1) !== dayCount) {
                return [{time: '00:00', level: lastTrig.level}];
            }
            else {
                let timeForPrevDay = this.cycleCopy.duration - timeEndLastTrig.diff(moment('23:59', 'HH:mm'), 'minutes');
                let timeForNewDay = timeEndLastTrig.diff(moment('23:59', 'HH:mm'), 'minutes');
                let ratio = timeForPrevDay / this.cycleCopy.duration;
                let diffLevel = Math.round(lastTrig.level - predLastTrigLevel);
                let levelStartTrigger = Math.round(Number(predLastTrigLevel) + ratio * diffLevel);
                let start =  {time: '00:00', level: levelStartTrigger}
                let timeEndTrigger = moment('00:00', 'HH:mm').add(timeForNewDay, 'minutes').format('HH:mm');
                let end
                if (firstTrig.time > timeEndTrigger) {
                    end = {time: timeEndTrigger, level: lastTrig.level}
                }
                else {
                    let diffLevel = Math.round(  lastTrig.level - start.level  );
                    let ratio =  moment(firstTrig.time, 'HH:mm').diff( moment(start.time, 'HH:mm'), 'minutes') / timeForNewDay;
                    let levelEndTrigger = Math.round(Number(start.level) + ratio * diffLevel);
                    end = {time: firstTrig.time, level: levelEndTrigger}
                }
                return [
                    start, // start
                    end // end
                ];
            }
        },
        drawDayChart() {
            this.dayChart.data.datasets[0].data = this.getDayChartData(this.cycleCopy, this.currentDay.value);
            this.dayChart.update();
        },
        drawCycleChart() {
            let data = this.getCycleChartData(this.cycleCopy);
            this.cycleChart.data.datasets[0].data = data;
            this.cycleChart.options.scales.x.max = data[data.length - 1].date; // ограничиваем величиной последнего значения
            this.cycleChart.options.scales.x.ticks.maxTicksLimit = data.length;  // ограничим число отметок, чтоб не выводился следующий за концом периода день
            this.cycleChart.update();
        },
        saveCycle() {
            this.editModal.hide();
            this.cycleCopy.triggers = this.triggers;
            this.addOrEditCycle(this.cycleCopy);
        },
        insertTrigger(cycle, day) {
            // делаем копию, меняем ее и присваиваем обратно, чтоб отработал watcher
            let triggers = this.copyJson(this.triggers) || {};
            let index = this.getNextTriggerIndex(triggers);
            triggers[index] = {day: day.value, time: null, level: 0};
            this.triggers = triggers;
        },
        copyTriggersFromDay(day) {
            let dayTriggers = this.getDayTriggers(day.value);
            let triggers = this.copyJson(this.triggers);
            let index = this.getNextTriggerIndex(this.triggers);
            for (let trigger of dayTriggers) {
                triggers[index] = {
                    day: this.currentDay.value,
                    time: trigger.time,
                    level: trigger.level,
                };
                index++;
            }
            this.triggers = triggers;
        },
        deleteTrigger(trigger) {
            // делаем копию, удаляем, присваиваем, чтоб отработал watcher
            const id = trigger[0];
            let trigs = this.copyJson(this.triggers);
            delete trigs[id];
            this.triggers = trigs;
        },
        onNavi(incr) {
            let index = this.currentDay.value + incr;
            if (index >= 0 && index < this.cycleDays.length) {
                this.currentDay = this.cycleDays[index];
            }
        },
        chooseDayForCopy(event, day) {
            if (day.value == this.currentDay.value) {
                event.preventDefault();
                return;
            }
            this.dayForCopy = day;
        }
    },
    watch: {
        cycle: function (obj) {
            let vm = this;
            // сюда кладем копию
            vm.cycleCopy = vm.copyJson(obj);
            // сюда кладем ссылку
            vm.currentDay = vm.cycleDays[0] || vm.currentDay;
            vm.triggers = vm.cycleCopy.triggers;
        },
        cycleCopy: {
            handler(cp) {
                // ссылка для удобства
                // this.triggers = this.cycleCopy.triggers;
                if (cp.startLevel < 0) {
                    cp.startLevel = 0;
                } else if (cp.startLevel > 100) {
                    cp.startLevel = 100;
                }
                this.drawDayChart();
                this.drawCycleChart();
            },
            deep: true,
        },
        triggers: {
            handler() {
                this.drawDayChart();
                this.drawCycleChart();
            },
            deep: true,
        },
        currentDay: {
            handler() {
                this.drawDayChart();
                this.drawCycleChart();
            },
            deep: true,
            immediate: false,
        },
    },
    mounted() {
        this.editModal = new bootstrap.Modal('#edit-cycle', {});
        // графики
        this.cycleChart = new Chart(this.$refs.cycleChart, cycleChartParams);
        this.dayChart = new Chart(this.$refs.dayChart, dayChartParams);
    }
}
