//Defaults
const Primary = getComputedStyle(document.documentElement).getPropertyValue('--Primary');
const Secondary = getComputedStyle(document.documentElement).getPropertyValue('--Secondary_1');
const Neutral_300 = getComputedStyle(document.documentElement).getPropertyValue('--Neutral_300');
const Neutral_900 = getComputedStyle(document.documentElement).getPropertyValue('--Neutral_900');
const Neutral_0 = getComputedStyle(document.documentElement).getPropertyValue('--Neutral_0');
const Neutral_100 = getComputedStyle(document.documentElement).getPropertyValue('--Neutral_100');
const DefaultFontSize = getComputedStyle(document.body).fontSize;
const borderWidth = 5;
const pointRadius = 3;
const animationDuration = 200;

Chart.register(ChartDataLabels);

//CycleChart
let cycleChartParams = {
  type: 'line',
  data: {
    labels:[],
    datasets: [{
      label: 'Освещённость',
      stepped: false,
      data: [],
      backgroundColor: Neutral_0,
      borderWidth: borderWidth,
      borderColor: Primary,
      pointRadius: pointRadius,
      pointHoverRadius: 12,
      hoverBorderWidth: 8,
      datalabels: {
        display: false,
      }
    }]
  },
  options: {
    animation: {
      duration: animationDuration,
    },
    parsing: {
      xAxisKey: 'date',
      yAxisKey: 'level',
    },
    scales: {
      x: {
        offset: false,
        type: 'linear',
        min: 1,
        title: {
          display: true,
          color: Neutral_300,
          font: {
            family: "'Inter', 'Arial', 'sans-serif'",
            size: 16,
          },
          text:  'Дни'
        },
        ticks: {
          padding: 10,
          stepSize: 1,
          includeBounds: false,
          autoSkip: false,
          font: {
            family: "'Inter', 'Arial', 'sans-serif'",
            size: 16,
          },
        },
        grid: {
          color: Neutral_100
        },
      },
      y: {
        ticks: {
          padding: 10,
          callback: function(value) {
            return value + '%';
          },
          font: {
            family: "'Inter', 'Arial', 'sans-serif'",
            size: 16,
            color: Neutral_100
          }
        },
        grid: {
          color: Neutral_100
        },
        min: 0,
        max: 100
      }
    },
    responsive: true,
    plugins: {
      tooltip: {
        enabled: true,
        usePointStyle: true,
        titleFont: {
          family: "'Inter', 'Arial', 'sans-serif'",
          size: 18,
        },
        bodyFont: {
          family: "'Inter', 'Arial', 'sans-serif'",
          size: 14,
        },
        callbacks: {
          label: function(context) {
            let label = context.dataset.label || '';
            if (label) {
              label += ': ';
            };
            if (context.parsed.y !== null) {
              label += new Intl.NumberFormat().format(context.parsed.y);
            };
            return label + '%';
          },

          labelPointStyle: function() {
            return {
                pointStyle: 'circle',
                rotation: 0,
            };
          },

          title: function(context) {
            // переводим дни в "день D HH:mm"
            const sutki = 1440;  // число минут в сутках
            const oneHour = 60;
            let date = parseFloat(context[0].label.replace(',', '.'));
            const day = Math.trunc(date);
            let minute = (date - day) * sutki;
            let hour = Math.trunc(minute / oneHour);
            hour = hour.toString().padStart(2, '0');
            minute = Math.round(minute % 60);  // тут будут неточности
            minute = minute.toString().padStart(2, '0');
            return `День ${day}  ${hour}:${minute}`;
          },
        },
      },
      legend: false,
      elements: {
        line: {
          tension: 0
        },
      },
    }
  }
};

//DayChart
let dayChartParams = {
    type: 'line',
    data: {
      // labels: HoursCycleArr,
      datasets: [{
        label: 'Освещённость',
        stepped: false,
        data: [], //DataGraphDay,
        backgroundColor: Neutral_0,
        borderWidth: borderWidth,
        borderColor: Primary,
        pointRadius: pointRadius,
        pointHoverRadius: 12,
        hoverBorderWidth: 8,
        datalabels: {
          display: false,
        }
      }]
    },
    options: {
      animation: {
        duration: animationDuration,
      },
      parsing: {
        xAxisKey: 'time',
        yAxisKey: 'level',
      },
      scales: {
        x: {
          offset: false,
          type: 'time',
          time: {
              unit: 'hour',
              parser: 'HH:mm',
              displayFormats: {
                        hour: 'H'
                    }
          },
          min: '00:00',
          max: '23:59',
          title: {
            display: true,
            color: Neutral_300,
            font: {
              family: "'Inter', 'Arial', 'sans-serif'",
              size: 16,
            },
            text:  'Часы'
          },
          ticks: {
            source: 'auto',
            padding: 10,
            font: {
              family: "'Inter', 'Arial', 'sans-serif'",
              size: 12,
            }
          },
          grid: {
            color: Neutral_100
          },
        },
        y: {
          ticks: {
            padding: 10,
            callback: function(value) {
              return value + '%';
            },
            font: {
              family: "'Inter', 'Arial', 'sans-serif'",
              size: 16,
              color: Neutral_100
            }
          },
          grid: {
            color: Neutral_100
          },
          min: 0,
          max: 100
        }
      },
      responsive: true,
      plugins: {
        tooltip: {
          enabled: true,
          usePointStyle: true,
          titleFont: {
            family: "'Inter', 'Arial', 'sans-serif'",
            size: 18
          },
          bodyFont: {
            family: "'Inter', 'Arial', 'sans-serif'",
            size: 16,
          },
          callbacks: {
            label: function(context) {
              let label = context.dataset.label || '';
              if (label) {
                label += ': ';
              };
              if (context.parsed.y !== null) {
                label += new Intl.NumberFormat().format(context.parsed.y);
              };
              return label + '%';
            },

            labelPointStyle: function(context) {
              return {
                  pointStyle: 'circle',
                  rotation: 0,
              };
            },

            title: function(context) {
              return context[0].raw.time;
            },
          }
        },
        legend: false,
      },
      elements: {
        line: {
          tension: 0
        },
      },
    }
};
