from datetime import datetime

from spread_core.tools.utils import bytes2str
from .commands import Command
from .responses import Response


class Payload:
    mark = 'Payload'

    def __init__(self, _frame: bytes):
        self.frame = _frame

    def __repr__(self):
        return f'[{self.name}]: {bytes2str(self.frame)}'

    def to_dict(self):
        return dict(
            time=self.time,
            hex=bytes2str(self.frame),
            name=self.name
        )

    @property
    def time(self):
        time = datetime.now()
        return time.strftime('%H:%M:%S') + f'.{str(int(time.microsecond / 1000)).ljust(3, "0")}'

    @property
    def name(self) -> str: return self.__class__.__name__

    @property
    def desc(self) -> str: return ''


class ServicePacket(Payload):
    @property
    def name(self):
        arr = [self.command.__class__.__name__]
        if isinstance(self.command, Command):
            arr.append(bytes2str(self.command.params))
        elif isinstance(self.command, Response):
            arr.append(self.command.value)

        return ' '.join(str(i) for i in arr)
